///////////////////////////////////////////////////////////////////////////////
//
// Example Ericsson chatboard UART driver v0.01 beta!
// (c) 2003 Jouni 'Mr.Spiv/TRSi' Korhonen
//
//   http://www.deadcoderssociety.tk
//
// Note: this driver uses fully IRQ driven bidirectional
//       UART communication with the keyboard.
//       This code does not rely too much on the sdk.
//
//       The buffer handling etc in this driver is more or less
//       ad-hoc stuff typed in during one coding session so they might
//       contain some odd stuff :)
//
///////////////////////////////////////////////////////////////////////////////

#include <string.h>
#include "gp32.h"
#include "kbdrv.h"

//
//
//

static unsigned char _kbdBuffer[KBDBUFSIZE];
static volatile int _kbdBufferLo;
static volatile int _kbdBufferHi;
static volatile int _kbdBufferCnt;

static unsigned char _kbdTemp[KBDTMPSIZE];
static int _kbdTempCnt;

static unsigned char _kbdOut[KBDTMPSIZE];
static volatile int _kbdOutCnt;
static volatile int _kbdOutLen;
static int _mode = sms_mode;

static int sendACKZ( unsigned char * );


//
// few helper functions to get rid of the SDK
//

void installIRQ( int num, void (*irq)(void) ) {
	asm volatile(""
		"stmdb	sp!,{lr}	\n"
		"mov	r0,%[_num]	\n"
		"mov	r1,%[_irq]	\n"
		"swi	#0x09		\n"
		"ldmia	sp!,{lr}"
		: 
		: [_num] "r"(num), [_irq] "r"(irq)
		: "r0","r1");
}

//
//
//

static void removeIRQ( int num ) {
	asm volatile(""
		"stmdb	sp!,{lr}	\n"
		"mov	r0,%[_num]	\n"
		"swi	#0x0a		\n"
		"ldmia	sp!,{lr}"
		:
		: [_num] "r"(num)
		: "r0");
}

//
//
//

static void ARMDisableInt( void ) {
	asm volatile(""
		"mrs	r0,CPSR		\n"
		"orr	r0,r0,#192	\n"
		"msr	CPSR_fsxc,r0	\n"
		:
		:
		: "r0");
}

//
//
//

static void ARMEnableInt( void ) {
	asm volatile(""
		"mrs	r0,CPSR		\n"
		"bic	r0,r0,#128	\n"
		"msr	CPSR_fsxc,r0	\n"
		:
		:
		: "r0");
}

//
//
//
//
// UART0 Rx IRQ handler..
//
// More or less Ericsson specific IRQ handler.. as the
// keyboard only sends '\r' terminated ascii commands
// we just scan for them..
//
//

static void UART0RxInt(void) __attribute__ ((naked));
static void UART0RxInt(void) {
	asm volatile (
		"stmdb    r13!,{r0-r12,lr}"
	);
	long *p = (long *)PALETTE;

	static int foo = 0;
	unsigned char c;
	int cnt;
	
	
	cnt = rUFSTAT0 & 0x0f;
	cnt += ((rUFSTAT0 >> 8) & 0x01);

	*p = foo+=4;




	while (cnt-- > 0) {
		c = rURXH0;

		_kbdTemp[_kbdTempCnt++] = c;

		if (c == 13) {
			int i;

			// Check if we still have space for a new input..

			if (_kbdBufferCnt < KBDMAXCNT) {
				_kbdBufferCnt++;

				for (i = 0; i < _kbdTempCnt; i++) {
					_kbdBuffer[_kbdBufferHi++] = _kbdTemp[i];
					_kbdBufferHi = _kbdBufferHi % KBDBUFSIZE;
				}
	
				// Send ACK

				sendACKZ("OK\r\n");
			}

			_kbdTempCnt = 0;
		}
	}

	asm volatile (
		"ldmia  r13!,{r0-r12,lr}        \n"
		"subs pc,lr,#4"
	);
}

//
// More or less Ericsson specific IRQ handler.. as the
// ACK string must be zero terminated "OK\r\n" this
// handler has been made to do so..
//
//

static void UART0TxInt(void) __attribute__ ((naked));
static void UART0TxInt(void) {
	asm volatile (
		"stmdb    r13!,{r0-r12,lr}"
	);

	while (_kbdOutCnt < _kbdOutLen) {
		if (((rUFSTAT0 >> 4) & 0x0f) <= 15) {
			rUTXH0 = _kbdOut[_kbdOutCnt++];
		} else {
			break;
		}
	}

	if (_kbdOutCnt == _kbdOutLen) {
		_kbdOutLen = 0;
		_kbdOutCnt = 0;
		//rUCON0 &= ~4;
	}

	asm volatile (
		"ldmia  r13!,{r0-r12,lr}        \n"
		"subs pc,lr,#4"
	);
}

//
// Note:
//  Called within an interrupt..

int sendACKZ( unsigned char *s ) {
#if 1
	while (_kbdOutLen > 0);
	_kbdOutLen = strlen(s) + 1;
	_kbdOutCnt = 0;
	memcpy(_kbdOut,s,_kbdOutLen);		// adds the '\0' also..

	// Start writing..
	rSRCPND = (1 << 28);
	rINTPND = (1 << 28);

	while (_kbdOutCnt < _kbdOutLen) {
		if (((rUFSTAT0 >> 4) & 0x0f) <= 15) {
			rUTXH0 = _kbdOut[_kbdOutCnt++];
		} else {
			break;
		}
	}
#else
	do {
		while ((rUTRSTAT0 & 0x2) == 0);
		rUTXH0 = *s++;
	} while (*s);
#endif

	return 0;
}


//
//
// Returns:
//   0 if completed, 1 if postInitKbd() needs to be called..
//
//

int preInitKbd(long baud, long pclk) {
	unsigned char buf[256];
	int n;

	// Set up UART0
	rCLKCON		|= 0x100;
	rULCON0		= 0x03;	// 8 bits, no-parity
	rUCON0		= 0x85;	// polled mode & Rx timeout irq
	rUFCON0		= 0x37;	// enable & reset FIFOs, Rx 16 bytes, Tx 0 bytes
	rUBRDIV0	= (unsigned int)(pclk / (baud * 16)) - 1;

	// set up variables..

	_kbdBufferLo  = 0;
	_kbdBufferHi  = 0;
	_kbdBufferCnt = 0;

	_kbdTempCnt   = 0;
	_kbdOutCnt    = 0;
	_kbdOutLen    = 0;
	_mode         = sms_mode;

	// set up IRQs

	installIRQ(23,UART0RxInt);	// Rx int
	installIRQ(28,UART0TxInt);	// Tx int

	//
	// Start the chatboard negotiation..
	//


	// -> AT<13>
	getKbdRaw(buf,256);

	// If the first received data was not AT<13> then the
	// keyboard is already intialized.. no further init is
	// required. Note that this string is lost..

	if (strcmp(buf,"AT\r")) { return 0; }

	// -> AT+CGMM<13>
	getKbdRaw(buf,256);
	if (strcmp(buf,"AT+CGMM\r")) { return -1; }

	// -> AT+CGMR<13>
	getKbdRaw(buf,256);
	if (strcmp(buf,"AT+CGMR\r")) { return -2; }

	return 1;
}


//
//
//

int postInitKbd( void ) {
	unsigned char str[256];

	// -> AT*EDME?<13>
	getKbdRaw(str,256);
	if (strcmp(str,"AT*EDME?\r")) { return -1; }

	// -> AT*ESVM?<13>
	getKbdRaw(str,256);
	if (strcmp(str,"AT*ESVM?\r")) { return -2; }


	// -> AT+CKPD="eseee<see<<s>>ss1"<13>
	getKbdRaw(str,256);
	if (strcmp(str,"AT+CKPD=\"eseee<see<<s>>ss1\"\r")) { return -3; }


	// -> AT+CKPD="c",25<13>
	getKbdRaw(str,256);
	if (strcmp(str,"AT+CKPD=\"c\",25\r")) { return -4; }

	return 0;
}

//
//
//

void removeKbd( void ) {
	removeIRQ(23);	// Rx int
	removeIRQ(28);	// Tx int
}


//
//
//

static int resolveKeyMapping( unsigned char *s ) {
	static int state = state_none;
	int n;

	// AT+CKPD="",20  -> num
	// AT+CKPD=""     -> char

	// www-key
	// AT*EDME?
	// AT*ESVM?
	// AT+CKPD="eseee<see<<s>>ss1"
	// AT+CKPD="c",25
	// AT+CKPD="91c91c911111110c"

	// Attachment key
	// AT+CKPD="#11111110c"

	// email-key
	// AT*EDME?
	// AT*ESVM?
	// AT+CKPD="eseee<see<<s>>ss1"
	// AT+CKPD="c",25
	// AT+CKPD="8666*1111111*0c"

	// sms-key
	// AT*EDME?
	// AT*ESVM?
	// AT+CKPD="eseee<see<<s>>ss1"
	// AT+CKPD="c",25

	// phonebool-key
	// AT*EDME?
	// AT*ESVM?
	// AT+CKPD="eseee<see<<s>>ss"

	return 0;
}


//
//
//

int waitKbdKey( void ) {
	unsigned char buf[256];
	unsigned char c;
	int n = 0;

	if (_kbdBufferLo == _kbdBufferHi) {
		return 0;
	}

	ARMDisableInt();

	while (n < 256) {
		buf[n++] = c = _kbdBuffer[_kbdBufferLo++];
		_kbdBufferLo = _kbdBufferLo % KBDBUFSIZE;
		if (c == 13) { 
			buf[n] = '\0';
			break;
		}
	}
	_kbdBufferCnt--;


	//
	// find out the character..
	//

	
	ARMEnableInt();
	return 0;
}

//
//
//

int pollKbdKey( void ) {

	return 0;
}

//
// Note:
//  This call blocks.
//

int getKbdRaw( unsigned char *buf, int len ) {
	unsigned char c;
	int n = 0;

	while (_kbdBufferLo == _kbdBufferHi);
	ARMDisableInt();

	while (n < len) {
		buf[n++] = c = _kbdBuffer[_kbdBufferLo++];
		_kbdBufferLo = _kbdBufferLo % KBDBUFSIZE;
		if (c == 13) { 
			buf[n] = '\0';
			break;
		}
	}
	_kbdBufferCnt--;

	ARMEnableInt();
	return n;
}

//
// Note:
//  This call is non-blocking.
//

int getKbdRawNB( unsigned char *buf, int len ) {
	unsigned char c;
	int n = 0;

	if (_kbdBufferLo == _kbdBufferHi) {
		return 0;
	}

	ARMDisableInt();

	while (n < len) {
		buf[n++] = c = _kbdBuffer[_kbdBufferLo++];
		_kbdBufferLo = _kbdBufferLo % KBDBUFSIZE;
		if (c == 13) { 
			buf[n] = '\0';
			break;
		}
	}
	_kbdBufferCnt--;

	ARMEnableInt();
	return n;
}
