/*
 * main_GP32.i
 *
 */
#include "Version.h"

extern int init_graphics(void);

extern "C" {
#include "gp32/gpmisc.h"
//int cpu_speed(int master_speed, int div_factor, int clk_mode);
}

#include "pogo.h"

// Global variables
char Frodo::prefs_path[256] = "";


/*
 *  Create application object and start it
 */

int main(int argc, char **argv)
{
	Frodo *the_app;
	char *args[]={ "Frodo", NULL };
	DIR *frododir;


	// fs test routine data
	#define DATA_LENGTH 35000
	char *test_data;
	char *test_data2;
	int i;
	char x;
	FILE *test_file;
	// end fs test routine data

	pogo_init();
	timer_init();
	key_init();
	console_init();
	filesys_init();
	//sound_init();

	ioctl(1, IO_SETMODE, CM_LINEWRAP);

        //cpu_speed(67500000, (37<<12) | (0<<4) | 2, 2);
	//cpu_speed(102000000, (43<<12) | (1<<4) | 1, 2);
	//cpu_speed(133500000, (81<<12) | (2<<4) | 1, 2);

	printf("%s by Christian Bauer\n", VERSION_STRING);
	printf("GP32 port by Mike Dawson\n");
	printf("\npress B to exit debug screen\n\n");

	// set up directories
	frododir=opendir("/smfs/gpmm/c64");
	if(frododir==NULL) {
		printf("Setting up data directories...\n");
		printf("This takes a little while the first time you run Frodo\n");
		printf("Next time it'll start up really speedily :)\n\n");
		mkdir("/smfs/gpmm/c64");
	} else {
		closedir(frododir);
	}
	frododir=opendir("/smfs/gpmm/c64/frodo");
	if(frododir==NULL) {
		mkdir("/smfs/gpmm/c64/frodo");
	} else {
		closedir(frododir);
	}

	// fs test routine
	/*
	test_data=(char *)malloc(DATA_LENGTH);
	test_data2=(char *)malloc(DATA_LENGTH);
	const char *test_filename="/smfs/tmp1";
	x=0;
	for(i=0; i<DATA_LENGTH; i++) {
		test_data[i]=x++;
	}
	printf("writing %d length file\n", DATA_LENGTH);
	remove(test_filename);
	test_file=fopen(test_filename, "w");
	if(!test_file) {
		printf("error opening file\n");
		while(1);
	}
	i=fwrite((void *)test_data, 1, DATA_LENGTH, test_file);
	fclose(test_file);
	printf("%d bytes written\n", i);

	printf("reading file back\n");
	test_file=fopen(test_filename, "r");
	if(!test_file) {
		printf("error opening file\n");
		while(1);
	}
	i=fread((void *)test_data2, 1, DATA_LENGTH, test_file);
	fclose(test_file);
	printf("%d bytes read\n", i);
	printf("comparing data\n");
	for(i=0; i<DATA_LENGTH; i++) {
		if(test_data[i]!=test_data2[i]) break;
	}
	if(i==DATA_LENGTH) {
		printf("data matches\n");
	} else {
		printf("data differs at %d bytes\n", i);
		while(1);
	}*/

	// end fs test routine
	
	if (!init_graphics())
		return 0;

	the_app = new Frodo();
	//the_app->ArgvReceived(argc, argv);
	the_app->ArgvReceived(1, args);

	ThePrefs.SkipFrames=1;
	//ThePrefs.SIDFilters=false;
	ThePrefs.SIDType=SIDTYPE_DIGITAL;
	ThePrefs.LimitSpeed=true;

	the_app->ReadyToRun();
	delete the_app;

	printf("frodo terminated\n");
	while(1);
}


/*
 *  Constructor: Initialize member variables
 */

Frodo::Frodo()
{
	TheC64 = NULL;
}


/*
 *  Process command line arguments
 */

void Frodo::ArgvReceived(int argc, char **argv)
{
	if (argc == 2)
		strncpy(prefs_path, argv[1], 255);
}


/*
 *  Arguments processed, run emulation
 */

void Frodo::ReadyToRun(void)
{
	//getcwd(AppDirPath, 256);

	// Load preferences
	/*
	if (!prefs_path[0]) {
		char *home = getenv("HOME");
		if (home != NULL && strlen(home) < 240) {
			strncpy(prefs_path, home, 200);
			strcat(prefs_path, "/");
		}
		strcat(prefs_path, ".frodorc");
	}
	ThePrefs.Load(prefs_path);
	*/

	// Create and start C64
	TheC64 = new C64;
	if (load_rom_files())
		TheC64->Run();
	delete TheC64;
}


Prefs *Frodo::reload_prefs(void)
{
	static Prefs newprefs;
	newprefs.Load(prefs_path);
	return &newprefs;
}
